==> choreonoid_1.1.0+dfsg.orig.tar.gz: choreonoid-1.1.0+dfsg/src/Body/LinkPath.cpp <==
/**
   \file
   \brief Implementations of the LinkPath class
   \author Shin'ichiro Nakaoka
*/
  
#include "LinkPath.h"
#include "Link.h"
#include <algorithm>

using namespace std;
using namespace cnoid;


LinkPath::LinkPath()
{

}


LinkPath::LinkPath(Link* base, Link* end)
{
    find(base, end);
}


/// path from the root link
LinkPath::LinkPath(Link* base)
{
    find(base);
}


/// This method is disabled.
void LinkPath::find(Link* root, bool doUpward, bool doDownward)
{
    throw "The find method for LinkTraverse cannot be used in LinkPath";
}


bool LinkPath::find(Link* base, Link* end)
{
    links.clear();
    numUpwardConnections = 0;
    bool found = findPathSub(base, 0, end, false);
    if(!found){
        links.clear();
    }
    return found;
}


bool LinkPath::findPathSub(Link* link, Link* prev, Link* end, bool isUpward)
{
    links.push_back(link);
    if(isUpward){
        ++numUpwardConnections;
    }
    
    if(link == end){
        return true;
    }

    for(Link* child = link->child; child; child = child->sibling){
        if(child != prev){
            if(findPathSub(child, link, end, false)){
                return true;
            }
        }
    }

    Link* parent = link->parent;
    if(parent && parent != prev){
        if(findPathSub(parent, link, end, true)){
            return true;
        }
    }

    links.pop_back();
    if(isUpward){
        --numUpwardConnections;
    }

    return false;
}


/// path from the root link
void LinkPath::find(Link* end)
{
    links.clear();
    numUpwardConnections = 0;
    findPathFromRootSub(end);
    std::reverse(links.begin(), links.end());
}


void LinkPath::findPathFromRootSub(Link* link)
{
    links.push_back(link);
    if(link->parent){
        findPathFromRootSub(link->parent);
    }
}

==> choreonoid_1.5.0+dfsg.orig.tar.gz: choreonoid-1.5.0/src/Body/LinkPath.cpp <==
/**
   \file
   \brief Implementations of the LinkPath class
   \author Shin'ichiro Nakaoka
*/
  
#include "LinkPath.h"
#include "Link.h"
#include <algorithm>

using namespace std;
using namespace cnoid;


LinkPath::LinkPath()
{

}


LinkPath::LinkPath(Link* base, Link* end)
{
    setPath(base, end);
}


/// path from the root link
LinkPath::LinkPath(Link* base)
{
    setPath(base);
}


/// This method is disabled.
void LinkPath::find(Link* root, bool doUpward, bool doDownward)
{
    throw "The find method for LinkTraverse cannot be used in LinkPath";
}


bool LinkPath::setPath(Link* base, Link* end)
{
    links.clear();
    numUpwardConnections = 0;
    bool found = findPathSub(base, 0, end, false);
    if(!found){
        links.clear();
    }
    return found;
}


bool LinkPath::findPathSub(Link* link, Link* prev, Link* end, bool isUpward)
{
    links.push_back(link);
    if(isUpward){
        ++numUpwardConnections;
    }
    
    if(link == end){
        return true;
    }

    for(Link* child = link->child(); child; child = child->sibling()){
        if(child != prev){
            if(findPathSub(child, link, end, false)){
                return true;
            }
        }
    }

    Link* parent = link->parent();
    if(parent && parent != prev){
        if(findPathSub(parent, link, end, true)){
            return true;
        }
    }

    links.pop_back();
    if(isUpward){
        --numUpwardConnections;
    }

    return false;
}


/// path from the root link
void LinkPath::setPath(Link* end)
{
    links.clear();
    numUpwardConnections = 0;
    findPathFromRootSub(end);
    std::reverse(links.begin(), links.end());
}


void LinkPath::findPathFromRootSub(Link* link)
{
    links.push_back(link);
    if(link->parent()){
        findPathFromRootSub(link->parent());
    }
}
